@@ -1,6 +1,8 @@
 #!/usr/bin/env python2
 from __future__ import absolute_import
+from __future__ import division
 
+import argparse
 from pwn import *
 from pwnlib.commandline import common
 
@@ -11,7 +13,7 @@ parser = common.parser_commands.add_parser(
 parser.add_argument(
     'elf',
     nargs='+',
-    type=file,
+    type=argparse.FileType('rb'),
     help='Files to check'
 )
 
