@@ -1,14 +1,18 @@
 # -*- coding: utf-8 -*-
 from __future__ import absolute_import
+from __future__ import division
 
 import logging
 import re
+import six
 import string
 import subprocess
 import sys
 import threading
 import time
 
+from six.moves import range
+
 from pwnlib import atexit
 from pwnlib import term
 from pwnlib.context import context
@@ -30,7 +34,7 @@ class tube(Timeout, Logger):
 
     #: Delimiter to use for :meth:`sendline`, :meth:`recvline`,
     #: and related functions.
-    newline = '\n'
+    newline = b'\n'
 
     def __init__(self, timeout = default, level = None, *a, **kw):
         super(tube, self).__init__(timeout)
@@ -44,7 +48,7 @@ class tube(Timeout, Logger):
 
     # Functions based on functions from subclasses
     def recv(self, numb = None, timeout = default):
-        r"""recv(numb = 4096, timeout = default) -> str
+        r"""recv(numb = 4096, timeout = default) -> bytes
 
         Receives up to `numb` bytes of data from the tube, and returns
         as soon as any quantity of data is available.
@@ -56,26 +60,26 @@ class tube(Timeout, Logger):
             exceptions.EOFError: The connection is closed
 
         Returns:
-            A string containing bytes received from the socket,
+            A bytes object containing bytes received from the socket,
             or ``''`` if a timeout occurred while waiting.
 
         Examples:
 
             >>> t = tube()
             >>> # Fake a data source
-            >>> t.recv_raw = lambda n: 'Hello, world'
-            >>> t.recv() == 'Hello, world'
+            >>> t.recv_raw = lambda n: b'Hello, world'
+            >>> t.recv() == b'Hello, world'
             True
-            >>> t.unrecv('Woohoo')
-            >>> t.recv() == 'Woohoo'
+            >>> t.unrecv(b'Woohoo')
+            >>> t.recv() == b'Woohoo'
             True
             >>> with context.local(log_level='debug'):
             ...    _ = t.recv() # doctest: +ELLIPSIS
             [...] Received 0xc bytes:
-                'Hello, world'
+                b'Hello, world'
         """
         numb = self.buffer.get_fill_size(numb)
-        return self._recv(numb, timeout) or ''
+        return self._recv(numb, timeout) or b''
 
     def unrecv(self, data):
         """unrecv(data)
@@ -86,17 +90,18 @@ class tube(Timeout, Logger):
         Examples:
 
             >>> t = tube()
-            >>> t.recv_raw = lambda n: 'hello'
+            >>> t.recv_raw = lambda n: b'hello'
             >>> t.recv()
-            'hello'
+            b'hello'
             >>> t.recv()
-            'hello'
-            >>> t.unrecv('world')
+            b'hello'
+            >>> t.unrecv(b'world')
             >>> t.recv()
-            'world'
+            b'world'
             >>> t.recv()
-            'hello'
+            b'hello'
         """
+        data = context._encode(data)
         self.buffer.unget(data)
 
     def _fillbuffer(self, timeout = default):
@@ -112,15 +117,15 @@ class tube(Timeout, Logger):
         Examples:
 
             >>> t = tube()
-            >>> t.recv_raw = lambda *a: 'abc'
+            >>> t.recv_raw = lambda *a: b'abc'
             >>> len(t.buffer)
             0
             >>> t._fillbuffer()
-            'abc'
+            b'abc'
             >>> len(t.buffer)
             3
         """
-        data = ''
+        data = b''
 
         with self.local(timeout):
             data = self.recv_raw(self.buffer.get_fill_size())
@@ -130,7 +135,7 @@ class tube(Timeout, Logger):
 
             if len(set(data)) == 1 and len(data) > 1:
                 self.indented('%r * %#x' % (data[0], len(data)), level = logging.DEBUG)
-            elif all(c in string.printable for c in data):
+            elif all(c in string.printable.encode() for c in data):
                 for line in data.splitlines(True):
                     self.indented(repr(line), level = logging.DEBUG)
             else:
@@ -149,19 +154,18 @@ class tube(Timeout, Logger):
         buffer is empty.
         """
         numb = self.buffer.get_fill_size(numb)
-        data = ''
 
         # No buffered data, could not put anything in the buffer
         # before timeout.
         if not self.buffer and not self._fillbuffer(timeout):
-            return ''
+            return b''
 
         return self.buffer.get(numb)
 
     def recvpred(self, pred, timeout = default):
-        """recvpred(pred, timeout = default) -> str
+        """recvpred(pred, timeout = default) -> bytes
 
-        Receives one byte at a time from the tube, until ``pred(bytes)``
+        Receives one byte at a time from the tube, until ``pred(all_bytes)``
         evaluates to True.
 
         If the request is not satisfied before ``timeout`` seconds pass,
@@ -175,11 +179,11 @@ class tube(Timeout, Logger):
             exceptions.EOFError: The connection is closed
 
         Returns:
-            A string containing bytes received from the socket,
+            A bytes object containing bytes received from the socket,
             or ``''`` if a timeout occurred while waiting.
         """
 
-        data = ''
+        data = b''
 
         with self.countdown(timeout):
             while not pred(data):
@@ -187,13 +191,13 @@ class tube(Timeout, Logger):
                     res = self.recv(1)
                 except Exception:
                     self.unrecv(data)
-                    return ''
+                    return b''
 
                 if res:
                     data += res
                 else:
                     self.unrecv(data)
-                    return ''
+                    return b''
 
         return data
 
@@ -215,21 +219,21 @@ class tube(Timeout, Logger):
         Examples:
 
             >>> t = tube()
-            >>> data = 'hello world'
+            >>> data = b'hello world'
             >>> t.recv_raw = lambda *a: data
             >>> t.recvn(len(data)) == data
             True
-            >>> t.recvn(len(data)+1) == data + data[0]
+            >>> t.recvn(len(data)+1) == data + data[:1]
             True
             >>> t.recv_raw = lambda *a: None
             >>> # The remaining data is buffered
             >>> t.recv() == data[1:]
             True
-            >>> t.recv_raw = lambda *a: time.sleep(0.01) or 'a'
+            >>> t.recv_raw = lambda *a: time.sleep(0.01) or b'a'
             >>> t.recvn(10, timeout=0.05)
-            ''
+            b''
             >>> t.recvn(10, timeout=0.06) # doctest: +ELLIPSIS
-            'aaaaaa...'
+            b'aaaaaa...'
         """
         # Keep track of how much data has been received
         # It will be pasted together at the end if a
@@ -239,12 +243,12 @@ class tube(Timeout, Logger):
                 pass
 
         if len(self.buffer) < numb:
-            return ''
+            return b''
 
         return self.buffer.get(numb)
 
-    def recvuntil(self, delims, drop=False, timeout = default):
-        """recvuntil(delims, timeout = default) -> str
+    def recvuntil(self, delims, drop=False, timeout=default):
+        """recvuntil(delims, drop=False, timeout=default) -> bytes
 
         Receive data until one of `delims` is encountered.
 
@@ -252,7 +256,7 @@ class tube(Timeout, Logger):
         all data is buffered and an empty string (``''``) is returned.
 
         arguments:
-            delims(str,tuple): String of delimiters characters, or list of delimiter strings.
+            delims(bytes,tuple): Byte-string of delimiters characters, or list of delimiter byte-strings.
             drop(bool): Drop the ending.  If :const:`True` it is removed from the end of the return value.
 
         Raises:
@@ -265,51 +269,52 @@ class tube(Timeout, Logger):
         Examples:
 
             >>> t = tube()
-            >>> t.recv_raw = lambda n: "Hello World!"
-            >>> t.recvuntil(' ')
-            'Hello '
+            >>> t.recv_raw = lambda n: b"Hello World!"
+            >>> t.recvuntil(b' ')
+            b'Hello '
             >>> _=t.clean(0)
             >>> # Matches on 'o' in 'Hello'
-            >>> t.recvuntil(tuple(' Wor'))
-            'Hello'
+            >>> t.recvuntil((b' ',b'W',b'o',b'r'))
+            b'Hello'
             >>> _=t.clean(0)
             >>> # Matches expressly full string
-            >>> t.recvuntil(' Wor')
-            'Hello Wor'
+            >>> t.recvuntil(b' Wor')
+            b'Hello Wor'
             >>> _=t.clean(0)
             >>> # Matches on full string, drops match
-            >>> t.recvuntil(' Wor', drop=True)
-            'Hello'
+            >>> t.recvuntil(b' Wor', drop=True)
+            b'Hello'
 
             >>> # Try with regex special characters
             >>> t = tube()
-            >>> t.recv_raw = lambda n: "Hello|World"
-            >>> t.recvuntil('|', drop=True)
-            'Hello'
+            >>> t.recv_raw = lambda n: b"Hello|World"
+            >>> t.recvuntil(b'|', drop=True)
+            b'Hello'
 
         """
         # Convert string into singleton tupple
-        if isinstance(delims, (str, unicode)):
+        if isinstance(delims, (bytes, six.text_type)):
             delims = (delims,)
+        delims = tuple(map(context._encode, delims))
 
         # Longest delimiter for tracking purposes
         longest = max(map(len, delims))
 
         # Cumulative data to search
         data = []
-        top = ''
+        top = b''
 
         with self.countdown(timeout):
             while self.countdown_active():
                 try:
                     res = self.recv(timeout=self.timeout)
                 except Exception:
-                    self.unrecv(''.join(data) + top)
+                    self.unrecv(b''.join(data) + top)
                     raise
 
                 if not res:
-                    self.unrecv(''.join(data) + top)
-                    return ''
+                    self.unrecv(b''.join(data) + top)
+                    return b''
 
                 top += res
                 start = len(top)
@@ -324,16 +329,16 @@ class tube(Timeout, Logger):
                         top = top[:start]
                     else:
                         top = top[:end]
-                    return ''.join(data) + top
+                    return b''.join(data) + top
                 if len(top) > longest:
                     i = -longest - 1
                     data.append(top[:i])
                     top = top[i:]
 
-        return ''
+        return b''
 
-    def recvlines(self, numlines=2**20, keepends = False, timeout = default):
-        r"""recvlines(numlines, keepends = False, timeout = default) -> str list
+    def recvlines(self, numlines=2**20, keepends=False, timeout=default):
+        r"""recvlines(numlines, keepends=False, timeout=default) -> list of bytes objects
 
         Receive up to ``numlines`` lines.
 
@@ -358,25 +363,25 @@ class tube(Timeout, Logger):
         Examples:
 
             >>> t = tube()
-            >>> t.recv_raw = lambda n: '\n'
+            >>> t.recv_raw = lambda n: b'\n'
             >>> t.recvlines(3)
-            ['', '', '']
-            >>> t.recv_raw = lambda n: 'Foo\nBar\nBaz\n'
+            [b'', b'', b'']
+            >>> t.recv_raw = lambda n: b'Foo\nBar\nBaz\n'
             >>> t.recvlines(3)
-            ['Foo', 'Bar', 'Baz']
+            [b'Foo', b'Bar', b'Baz']
             >>> t.recvlines(3, True)
-            ['Foo\n', 'Bar\n', 'Baz\n']
+            [b'Foo\n', b'Bar\n', b'Baz\n']
         """
         lines = []
         with self.countdown(timeout):
-            for _ in xrange(numlines):
+            for _ in range(numlines):
                 try:
                     # We must set 'keepends' to True here so that we can
                     # restore the original, unmodified data to the buffer
                     # in the event of a timeout.
                     res = self.recvline(keepends=True, timeout=timeout)
                 except Exception:
-                    self.unrecv(''.join(lines))
+                    self.unrecv(b''.join(lines))
                     raise
 
                 if res:
@@ -389,8 +394,44 @@ class tube(Timeout, Logger):
 
         return lines
 
-    def recvline(self, keepends = True, timeout = default):
-        r"""recvline(keepends = True) -> str
+    def recvlinesS(self, numlines=2**20, keepends=False, timeout=default):
+        r"""recvlinesS(numlines, keepends=False, timeout=default) -> str list
+
+        This function is identical to :meth:`recvlines`, but decodes
+        the received bytes into string using :func:`context.encoding`.
+        You should use :meth:`recvlines` whenever possible for better performance.
+
+        Examples:
+
+            >>> t = tube()
+            >>> t.recv_raw = lambda n: b'\n'
+            >>> t.recvlinesS(3)
+            ['', '', '']
+            >>> t.recv_raw = lambda n: b'Foo\nBar\nBaz\n'
+            >>> t.recvlinesS(3)
+            ['Foo', 'Bar', 'Baz']
+        """
+        return [context._decode(x) for x in self.recvlines(numlines, keepends, timeout)]
+
+    def recvlinesb(self, numlines=2**20, keepends=False, timeout=default):
+        r"""recvlinesb(numlines, keepends=False, timeout=default) -> bytearray list
+
+        This function is identical to :meth:`recvlines`, but returns a bytearray.
+
+        Examples:
+
+            >>> t = tube()
+            >>> t.recv_raw = lambda n: b'\n'
+            >>> t.recvlinesb(3)
+            [bytearray(b''), bytearray(b''), bytearray(b'')]
+            >>> t.recv_raw = lambda n: b'Foo\nBar\nBaz\n'
+            >>> t.recvlinesb(3)
+            [bytearray(b'Foo'), bytearray(b'Bar'), bytearray(b'Baz')]
+        """
+        return [bytearray(x) for x in self.recvlines(numlines, keepends, timeout)]
+
+    def recvline(self, keepends=True, timeout=default):
+        r"""recvline(keepends=True, timeout=default) -> bytes
 
         Receive a single line from the tube.
 
@@ -412,21 +453,21 @@ class tube(Timeout, Logger):
         Examples:
 
             >>> t = tube()
-            >>> t.recv_raw = lambda n: 'Foo\nBar\r\nBaz\n'
+            >>> t.recv_raw = lambda n: b'Foo\nBar\r\nBaz\n'
             >>> t.recvline()
-            'Foo\n'
+            b'Foo\n'
             >>> t.recvline()
-            'Bar\r\n'
+            b'Bar\r\n'
             >>> t.recvline(keepends = False)
-            'Baz'
-            >>> t.newline = '\r\n'
+            b'Baz'
+            >>> t.newline = b'\r\n'
             >>> t.recvline(keepends = False)
-            'Foo\nBar'
+            b'Foo\nBar'
         """
         return self.recvuntil(self.newline, drop = not keepends, timeout = timeout)
 
-    def recvline_pred(self, pred, keepends = False, timeout = default):
-        r"""recvline_pred(pred, keepends = False) -> str
+    def recvline_pred(self, pred, keepends=False, timeout=default):
+        r"""recvline_pred(pred, keepends=False) -> bytes
 
         Receive data until ``pred(line)`` returns a truthy value.
         Drop all other data.
@@ -441,28 +482,28 @@ class tube(Timeout, Logger):
         Examples:
 
             >>> t = tube()
-            >>> t.recv_raw = lambda n: "Foo\nBar\nBaz\n"
-            >>> t.recvline_pred(lambda line: line == "Bar\n")
-            'Bar'
-            >>> t.recvline_pred(lambda line: line == "Bar\n", keepends=True)
-            'Bar\n'
-            >>> t.recvline_pred(lambda line: line == 'Nope!', timeout=0.1)
-            ''
+            >>> t.recv_raw = lambda n: b"Foo\nBar\nBaz\n"
+            >>> t.recvline_pred(lambda line: line == b"Bar\n")
+            b'Bar'
+            >>> t.recvline_pred(lambda line: line == b"Bar\n", keepends=True)
+            b'Bar\n'
+            >>> t.recvline_pred(lambda line: line == b'Nope!', timeout=0.1)
+            b''
         """
 
         tmpbuf = Buffer()
-        line   = ''
+        line   = b''
         with self.countdown(timeout):
             while self.countdown_active():
                 try:
                     line = self.recvline(keepends=True)
                 except Exception:
-                    self.buffer.add(tmpbuf)
+                    self.buffer.unget(tmpbuf)
                     raise
 
                 if not line:
-                    self.buffer.add(tmpbuf)
-                    return ''
+                    self.buffer.unget(tmpbuf)
+                    return b''
 
                 if pred(line):
                     if not keepends:
@@ -471,7 +512,7 @@ class tube(Timeout, Logger):
                 else:
                     tmpbuf.add(line)
 
-        return ''
+        return b''
 
     def recvline_contains(self, items, keepends = False, timeout = default):
         r"""
@@ -486,29 +527,30 @@ class tube(Timeout, Logger):
         Examples:
 
             >>> t = tube()
-            >>> t.recv_raw = lambda n: "Hello\nWorld\nXylophone\n"
-            >>> t.recvline_contains('r')
-            'World'
-            >>> f = lambda n: "cat dog bird\napple pear orange\nbicycle car train\n"
+            >>> t.recv_raw = lambda n: b"Hello\nWorld\nXylophone\n"
+            >>> t.recvline_contains(b'r')
+            b'World'
+            >>> f = lambda n: b"cat dog bird\napple pear orange\nbicycle car train\n"
             >>> t = tube()
             >>> t.recv_raw = f
-            >>> t.recvline_contains('pear')
-            'apple pear orange'
+            >>> t.recvline_contains(b'pear')
+            b'apple pear orange'
             >>> t = tube()
             >>> t.recv_raw = f
-            >>> t.recvline_contains(('car', 'train'))
-            'bicycle car train'
+            >>> t.recvline_contains((b'car', b'train'))
+            b'bicycle car train'
         """
-        if isinstance(items, (str,unicode)):
+        if isinstance(items, (bytes, six.text_type)):
             items = (items,)
+        items = tuple(map(context._encode, items))
 
         def pred(line):
             return any(d in line for d in items)
 
         return self.recvline_pred(pred, keepends, timeout)
 
-    def recvline_startswith(self, delims, keepends = False, timeout = default):
-        r"""recvline_startswith(delims, keepends = False, timeout = default) -> str
+    def recvline_startswith(self, delims, keepends=False, timeout=default):
+        r"""recvline_startswith(delims, keepends=False, timeout=default) -> bytes
 
         Keep receiving lines until one is found that starts with one of
         `delims`.  Returns the last line received.
@@ -527,24 +569,25 @@ class tube(Timeout, Logger):
         Examples:
 
             >>> t = tube()
-            >>> t.recv_raw = lambda n: "Hello\nWorld\nXylophone\n"
-            >>> t.recvline_startswith(tuple('WXYZ'))
-            'World'
-            >>> t.recvline_startswith(tuple('WXYZ'), True)
-            'Xylophone\n'
-            >>> t.recvline_startswith('Wo')
-            'World'
+            >>> t.recv_raw = lambda n: b"Hello\nWorld\nXylophone\n"
+            >>> t.recvline_startswith((b'W',b'X',b'Y',b'Z'))
+            b'World'
+            >>> t.recvline_startswith((b'W',b'X',b'Y',b'Z'), True)
+            b'Xylophone\n'
+            >>> t.recvline_startswith(b'Wo')
+            b'World'
         """
         # Convert string into singleton tupple
-        if isinstance(delims, (str, unicode)):
+        if isinstance(delims, (bytes, six.text_type)):
             delims = (delims,)
+        delims = tuple(map(context._encode, delims))
 
         return self.recvline_pred(lambda line: any(map(line.startswith, delims)),
                                   keepends=keepends,
                                   timeout=timeout)
 
-    def recvline_endswith(self, delims, keepends = False, timeout = default):
-        r"""recvline_endswith(delims, keepends = False, timeout = default) -> str
+    def recvline_endswith(self, delims, keepends=False, timeout=default):
+        r"""recvline_endswith(delims, keepends=False, timeout=default) -> bytes
 
         Keep receiving lines until one is found that starts with one of
         `delims`.  Returns the last line received.
@@ -557,26 +600,26 @@ class tube(Timeout, Logger):
         Examples:
 
             >>> t = tube()
-            >>> t.recv_raw = lambda n: 'Foo\nBar\nBaz\nKaboodle\n'
-            >>> t.recvline_endswith('r')
-            'Bar'
-            >>> t.recvline_endswith(tuple('abcde'), True)
-            'Kaboodle\n'
-            >>> t.recvline_endswith('oodle')
-            'Kaboodle'
+            >>> t.recv_raw = lambda n: b'Foo\nBar\nBaz\nKaboodle\n'
+            >>> t.recvline_endswith(b'r')
+            b'Bar'
+            >>> t.recvline_endswith((b'a',b'b',b'c',b'd',b'e'), True)
+            b'Kaboodle\n'
+            >>> t.recvline_endswith(b'oodle')
+            b'Kaboodle'
         """
         # Convert string into singleton tupple
-        if isinstance(delims, (str, unicode)):
+        if isinstance(delims, (bytes, six.text_type)):
             delims = (delims,)
 
-        delims = tuple(delim + self.newline for delim in delims)
+        delims = tuple(context._encode(delim) + self.newline for delim in delims)
 
         return self.recvline_pred(lambda line: any(map(line.endswith, delims)),
                                   keepends=keepends,
                                   timeout=timeout)
 
-    def recvregex(self, regex, exact = False, timeout = default):
-        """recvregex(regex, exact = False, timeout = default) -> str
+    def recvregex(self, regex, exact=False, timeout=default):
+        """recvregex(regex, exact=False, timeout=default) -> bytes
 
         Wrapper around :func:`recvpred`, which will return when a regex
         matches the string in the buffer.
@@ -588,7 +631,8 @@ class tube(Timeout, Logger):
         all data is buffered and an empty string (``''``) is returned.
         """
 
-        if isinstance(regex, (str, unicode)):
+        if isinstance(regex, (bytes, six.text_type)):
+            regex = context._encode(regex)
             regex = re.compile(regex)
 
         if exact:
@@ -598,8 +642,8 @@ class tube(Timeout, Logger):
 
         return self.recvpred(pred, timeout = timeout)
 
-    def recvline_regex(self, regex, exact = False, keepends = False, timeout = default):
-        """recvregex(regex, exact = False, keepends = False, timeout = default) -> str
+    def recvline_regex(self, regex, exact=False, keepends=False, timeout=default):
+        """recvline_regex(regex, exact=False, keepends=False, timeout=default) -> bytes
 
         Wrapper around :func:`recvline_pred`, which will return when a regex
         matches a line.
@@ -611,7 +655,8 @@ class tube(Timeout, Logger):
         all data is buffered and an empty string (``''``) is returned.
         """
 
-        if isinstance(regex, (str, unicode)):
+        if isinstance(regex, (bytes, six.text_type)):
+            regex = context._encode(regex)
             regex = re.compile(regex)
 
         if exact:
@@ -621,28 +666,28 @@ class tube(Timeout, Logger):
 
         return self.recvline_pred(pred, keepends = keepends, timeout = timeout)
 
-    def recvrepeat(self, timeout = default):
-        """recvrepeat(timeout = default) -> str
+    def recvrepeat(self, timeout=default):
+        """recvrepeat(timeout=default) -> bytes
 
         Receives data until a timeout or EOF is reached.
 
         Examples:
 
             >>> data = [
-            ... 'd',
-            ... '', # simulate timeout
-            ... 'c',
-            ... 'b',
-            ... 'a',
+            ... b'd',
+            ... b'', # simulate timeout
+            ... b'c',
+            ... b'b',
+            ... b'a',
             ... ]
             >>> def delayrecv(n, data=data):
             ...     return data.pop()
             >>> t = tube()
             >>> t.recv_raw = delayrecv
             >>> t.recvrepeat(0.2)
-            'abc'
+            b'abc'
             >>> t.recv()
-            'd'
+            b'd'
         """
 
         try:
@@ -654,7 +699,7 @@ class tube(Timeout, Logger):
         return self.buffer.get()
 
     def recvall(self, timeout=Timeout.forever):
-        """recvall() -> str
+        """recvall() -> bytes
 
         Receives data until EOF is reached.
         """
@@ -688,41 +733,45 @@ class tube(Timeout, Logger):
 
         Examples:
 
-            >>> def p(x): print repr(x)
+            >>> def p(x): print(repr(x))
             >>> t = tube()
             >>> t.send_raw = p
-            >>> t.send('hello')
-            'hello'
+            >>> t.send(b'hello')
+            b'hello'
         """
 
+        data = context._encode(data)
+
         if self.isEnabledFor(logging.DEBUG):
             self.debug('Sent %#x bytes:' % len(data))
             if len(set(data)) == 1:
                 self.indented('%r * %#x' % (data[0], len(data)))
-            elif all(c in string.printable for c in data):
+            elif all(c in string.printable.encode() for c in data):
                 for line in data.splitlines(True):
                     self.indented(repr(line), level = logging.DEBUG)
             else:
                 self.indented(fiddling.hexdump(data), level = logging.DEBUG)
         self.send_raw(data)
 
-    def sendline(self, line=''):
+    def sendline(self, line=b''):
         r"""sendline(data)
 
         Shorthand for ``t.send(data + t.newline)``.
 
         Examples:
 
-            >>> def p(x): print repr(x)
+            >>> def p(x): print(repr(x))
             >>> t = tube()
             >>> t.send_raw = p
-            >>> t.sendline('hello')
-            'hello\n'
-            >>> t.newline = '\r\n'
-            >>> t.sendline('hello')
-            'hello\r\n'
+            >>> t.sendline(b'hello')
+            b'hello\n'
+            >>> t.newline = b'\r\n'
+            >>> t.sendline(b'hello')
+            b'hello\r\n'
         """
 
+        line = context._encode(line)
+
         self.send(line + self.newline)
 
     def sendlines(self, lines=[]):
@@ -732,37 +781,37 @@ class tube(Timeout, Logger):
     def sendafter(self, delim, data, timeout = default):
         """sendafter(delim, data, timeout = default) -> str
 
-        A combination of ``recvuntil(delim, timeout)`` and ``send(data)``.
+        A combination of ``recvuntil(delim, timeout=timeout)`` and ``send(data)``.
         """
 
-        res = self.recvuntil(delim, timeout)
+        res = self.recvuntil(delim, timeout=timeout)
         self.send(data)
         return res
 
     def sendlineafter(self, delim, data, timeout = default):
         """sendlineafter(delim, data, timeout = default) -> str
 
-        A combination of ``recvuntil(delim, timeout)`` and ``sendline(data)``."""
+        A combination of ``recvuntil(delim, timeout=timeout)`` and ``sendline(data)``."""
 
-        res = self.recvuntil(delim, timeout)
+        res = self.recvuntil(delim, timeout=timeout)
         self.sendline(data)
         return res
 
     def sendthen(self, delim, data, timeout = default):
         """sendthen(delim, data, timeout = default) -> str
 
-        A combination of ``send(data)`` and ``recvuntil(delim, timeout)``."""
+        A combination of ``send(data)`` and ``recvuntil(delim, timeout=timeout)``."""
 
         self.send(data)
-        return self.recvuntil(delim, timeout)
+        return self.recvuntil(delim, timeout=timeout)
 
     def sendlinethen(self, delim, data, timeout = default):
         """sendlinethen(delim, data, timeout = default) -> str
 
-        A combination of ``sendline(data)`` and ``recvuntil(delim, timeout)``."""
+        A combination of ``sendline(data)`` and ``recvuntil(delim, timeout=timeout)``."""
 
-        self.send(data + self.newline)
-        return self.recvuntil(delim, timeout)
+        self.sendline(data)
+        return self.recvuntil(delim, timeout=timeout)
 
     def interactive(self, prompt = term.text.bold_red('$') + ' '):
         """interactive(prompt = pwnlib.term.text.bold_red('$') + ' ')
@@ -782,10 +831,13 @@ class tube(Timeout, Logger):
             while not go.isSet():
                 try:
                     cur = self.recv(timeout = 0.05)
-                    cur = cur.replace('\r\n', '\n')
+                    cur = cur.replace(self.newline, b'\n')
                     if cur:
-                        sys.stdout.write(cur)
-                        sys.stdout.flush()
+                        stdout = sys.stdout
+                        if not term.term_mode:
+                            stdout = getattr(stdout, 'buffer', stdout)
+                        stdout.write(cur)
+                        stdout.flush()
                 except EOFError:
                     self.info('Got EOF while reading in interactive')
                     break
@@ -799,7 +851,8 @@ class tube(Timeout, Logger):
                 if term.term_mode:
                     data = term.readline.readline(prompt = prompt, float = True)
                 else:
-                    data = sys.stdin.read(1)
+                    stdin = getattr(sys.stdin, 'buffer', sys.stdin)
+                    data = stdin.read(1)
 
                 if data:
                     try:
@@ -823,7 +876,7 @@ class tube(Timeout, Logger):
 
         Similar to :func:`interactive`, except that no input is sent.
 
-        Similar to ``print tube.recvall()`` except that data is printed
+        Similar to ``print(tube.recvall())`` except that data is printed
         as it is received, rather than after all data is received.
 
         Arguments:
@@ -837,7 +890,10 @@ class tube(Timeout, Logger):
         try:
             while True:
                 buf.add(function())
-                sys.stdout.write(buf.data[-1])
+                stdout = sys.stdout
+                if not term.term_mode:
+                    stdout = getattr(stdout, 'buffer', stdout)
+                stdout.write(buf.data[-1])
         except KeyboardInterrupt:
             pass
         except EOFError:
@@ -863,9 +919,9 @@ class tube(Timeout, Logger):
         Examples:
 
             >>> t = tube()
-            >>> t.unrecv('clean me up')
+            >>> t.unrecv(b'clean me up')
             >>> t.clean(0)
-            'clean me up'
+            b'clean me up'
             >>> len(t.buffer)
             0
         """
@@ -886,7 +942,7 @@ class tube(Timeout, Logger):
 
         Examples:
 
-            >>> def recv(n, data=['', 'hooray_data']):
+            >>> def recv(n, data=[b'', b'hooray_data']):
             ...     while data: return data.pop()
             >>> t = tube()
             >>> t.recv_raw      = recv
@@ -895,9 +951,9 @@ class tube(Timeout, Logger):
             >>> with context.local(log_level='info'):
             ...     data = t.clean_and_log() #doctest: +ELLIPSIS
             [DEBUG] Received 0xb bytes:
-                'hooray_data'
+                b'hooray_data'
             >>> data
-            'hooray_data'
+            b'hooray_data'
             >>> context.clear()
         """
         with context.local(log_level='debug'):
@@ -911,8 +967,8 @@ class tube(Timeout, Logger):
 
         Examples:
 
-            >>> def p(x): print x
-            >>> def recvone(n, data=['data']):
+            >>> def p(x): print(x.decode())
+            >>> def recvone(n, data=[b'data']):
             ...     while data: return data.pop()
             ...     raise EOFError
             >>> a = tube()
@@ -967,8 +1023,8 @@ class tube(Timeout, Logger):
 
         Examples:
 
-            >>> def p(x): print x
-            >>> def recvone(n, data=['data']):
+            >>> def p(x): print(repr(x))
+            >>> def recvone(n, data=[b'data']):
             ...     while data: return data.pop()
             ...     raise EOFError
             >>> a = tube()
@@ -980,7 +1036,7 @@ class tube(Timeout, Logger):
             >>> a.shutdown      = lambda d: True
             >>> b.shutdown      = lambda d: True
             >>> _=(a.connect_output(b), time.sleep(0.1))
-            data
+            b'data'
         """
 
         other.connect_input(self)
@@ -1050,11 +1106,14 @@ class tube(Timeout, Logger):
         """
         self << other << self
 
-    def wait_for_close(self):
+    def wait_for_close(self, timeout=default):
         """Waits until the tube is closed."""
 
-        while self.connected():
-            time.sleep(0.05)
+        with self.countdown(timeout):
+            while self.countdown_active():
+                if not self.connected():
+                    return
+                time.sleep(min(self.timeout, 0.05))
 
     wait = wait_for_close
 
@@ -1070,7 +1129,7 @@ class tube(Timeout, Logger):
             >>> t.can_recv_raw = lambda *a: False
             >>> t.can_recv()
             False
-            >>> _=t.unrecv('data')
+            >>> _=t.unrecv(b'data')
             >>> t.can_recv()
             True
             >>> _=t.recv()
@@ -1126,10 +1185,10 @@ class tube(Timeout, Logger):
 
         Examples:
 
-            >>> def p(x): print x
+            >>> def p(x): print(x)
             >>> t = tube()
             >>> t.shutdown_raw = p
-            >>> _=map(t.shutdown, ('in', 'read', 'recv', 'out', 'write', 'send'))
+            >>> _=list(map(t.shutdown, ('in', 'read', 'recv', 'out', 'write', 'send')))
             recv
             recv
             recv
@@ -1159,10 +1218,10 @@ class tube(Timeout, Logger):
 
         Doctest:
 
-            >>> def p(x): print x
+            >>> def p(x): print(x)
             >>> t = tube()
             >>> t.connected_raw = p
-            >>> _=map(t.connected, ('any', 'in', 'read', 'recv', 'out', 'write', 'send'))
+            >>> _=list(map(t.connected, ('any', 'in', 'read', 'recv', 'out', 'write', 'send')))
             any
             recv
             recv
@@ -1188,7 +1247,7 @@ class tube(Timeout, Logger):
         Examples:
 
             >>> t = tube()
-            >>> def p(x): print x
+            >>> def p(x): print(x)
             >>> t.close = lambda: p("Closed!")
             >>> with t: pass
             Closed!
@@ -1295,47 +1354,6 @@ class tube(Timeout, Logger):
 
         raise NotImplementedError()
 
-    #: Alias for :meth:`recv`
-    def read(self, *a, **kw): return self.recv(*a, **kw)
-    #: Alias for :meth:`recvpred`
-    def readpred(self, *a, **kw): return self.recvpred(*a, **kw)
-    #: Alias for :meth:`recvn`
-    def readn(self, *a, **kw): return self.recvn(*a, **kw)
-    #: Alias for :meth:`recvuntil`
-    def readuntil(self, *a, **kw): return self.recvuntil(*a, **kw)
-    #: Alias for :meth:`recvlines`
-    def readlines(self, *a, **kw): return self.recvlines(*a, **kw)
-    #: Alias for :meth:`recvline`
-    def readline(self, *a, **kw): return self.recvline(*a, **kw)
-    #: Alias for :meth:`recvline_pred`
-    def readline_pred(self, *a, **kw): return self.recvline_pred(*a, **kw)
-    #: Alias for :meth:`recvline_contains`
-    def readline_contains(self, *a, **kw): return self.recvline_contains(*a, **kw)
-    #: Alias for :meth:`recvline_startswith`
-    def readline_startswith(self, *a, **kw): return self.recvline_startswith(*a, **kw)
-    #: Alias for :meth:`recvline_endswith`
-    def readline_endswith(self, *a, **kw): return self.recvline_endswith(*a, **kw)
-    #: Alias for :meth:`recvregex`
-    def readregex(self, *a, **kw): return self.recvregex(*a, **kw)
-    #: Alias for :meth:`recvline_regex`
-    def readline_regex(self, *a, **kw): return self.recvline_regex(*a, **kw)
-    #: Alias for :meth:`recvrepeat`
-    def readrepeat(self, *a, **kw): return self.recvrepeat(*a, **kw)
-    #: Alias for :meth:`recvall`
-    def readall(self, *a, **kw): return self.recvall(*a, **kw)
-
-    #: Alias for :meth:`send`
-    def write(self, *a, **kw): return self.send(*a, **kw)
-    #: Alias for :meth:`sendline`
-    def writeline(self, *a, **kw): return self.sendline(*a, **kw)
-    #: Alias for :meth:`sendafter`
-    def writeafter(self, *a, **kw): return self.sendafter(*a, **kw)
-    #: Alias for :meth:`sendlineafter`
-    def writelineafter(self, *a, **kw): return self.sendlineafter(*a, **kw)
-    #: Alias for :meth:`sendthen`
-    def writethen(self, *a, **kw): return self.sendthen(*a, **kw)
-    #: Alias for :meth:`sendlinethen`
-    def writelinethen(self, *a, **kw): return self.sendlinethen(*a, **kw)
 
     def p64(self, *a, **kw):        return self.send(packing.p64(*a, **kw))
     def p32(self, *a, **kw):        return self.send(packing.p32(*a, **kw))
@@ -1351,3 +1369,52 @@ class tube(Timeout, Logger):
 
     def flat(self, *a, **kw):       return self.send(packing.flat(*a,**kw))
     def fit(self, *a, **kw):        return self.send(packing.fit(*a, **kw))
+
+    # Dynamic functions
+
+    def make_wrapper(func):
+        def wrapperb(self, *a, **kw):
+            return bytearray(func(self, *a, **kw))
+        def wrapperS(self, *a, **kw):
+            return context._decode(func(self, *a, **kw))
+        wrapperb.__doc__ = 'Same as :meth:`{func.__name__}`, but returns a bytearray'.format(func=func)
+        wrapperb.__name__ = func.__name__ + 'b'
+        wrapperS.__doc__ = 'Same as :meth:`{func.__name__}`, but returns a str, ' \
+                           'decoding the result using `context.encoding`. ' \
+                           '(note that the binary versions are way faster)'.format(func=func)
+        wrapperS.__name__ = func.__name__ + 'S'
+        return wrapperb, wrapperS
+
+    for func in [recv,
+                 recvn,
+                 recvall,
+                 recvrepeat,
+                 recvuntil,
+                 recvpred,
+                 recvregex,
+                 recvline,
+                 recvline_contains,
+                 recvline_startswith,
+                 recvline_endswith,
+                 recvline_regex]:
+        for wrapper in make_wrapper(func):
+            locals()[wrapper.__name__] = wrapper
+
+    def make_wrapper(func, alias):
+        def wrapper(self, *a, **kw):
+            return func(self, *a, **kw)
+        wrapper.__doc__ = 'Alias for :meth:`{func.__name__}`'.format(func=func)
+        wrapper.__name__ = alias
+        return wrapper
+
+    for _name in list(locals()):
+        if 'recv' in _name:
+            _name2 = _name.replace('recv', 'read')
+        elif 'send' in _name:
+            _name2 = _name.replace('send', 'write')
+        else:
+            continue
+        locals()[_name2] = make_wrapper(locals()[_name], _name2)
+
+    # Clean up the scope
+    del wrapper, func, make_wrapper, _name, _name2
