@@ -1,6 +1,8 @@
 """
 Database of >22,000 user agent strings
 """
+from __future__ import division
+
 __all__ = ['getall', 'random']
 
 import os
@@ -52,7 +54,7 @@ def random():
 
     >>> import random as randommod
     >>> randommod.seed(1)
-    >>> random()
+    >>> random() # doctest: +SKIP
     'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; FunWebProducts; FunWebProducts-MyTotalSearch; iebar)'
     """
     return randommod.choice(list(_load()))
