@@ -1,10 +1,12 @@
 #!/usr/bin/env python2
 from __future__ import absolute_import
+from __future__ import division
 
 import argparse
 import sys
 
 from pwnlib.commandline import common
+from pwnlib.util.fiddling import enhex
 
 parser = common.parser_commands.add_parser(
     'hex',
@@ -16,9 +18,12 @@ parser.add_argument('data', nargs='*',
 
 def main(args):
     if not args.data:
-        print sys.stdin.read().encode('hex')
+        print(enhex(getattr(sys.stdin, 'buffer', sys.stdin).read()))
     else:
-        print ' '.join(args.data).encode('hex')
+        data = ' '.join(args.data)
+        if not hasattr(data, 'decode'):
+            data = data.encode('utf-8', 'surrogateescape')
+        print(enhex(data))
 
 if __name__ == '__main__':
-    pwnlib.commandline.common.main(__file__)
+    common.main(__file__)
