@@ -5,12 +5,14 @@ Implements context management so that nested/scoped contexts and threaded
 contexts work properly and as expected.
 """
 from __future__ import absolute_import
+from __future__ import division
 
 import collections
 import functools
 import logging
 import os
 import platform
+import six
 import socket
 import stat
 import string
@@ -25,6 +27,11 @@ from pwnlib.config import register_config
 from pwnlib.device import Device
 from pwnlib.timeout import Timeout
 
+try:
+    from collections.abc import Iterable
+except ImportError:
+    from collections import Iterable
+
 __all__ = ['context', 'ContextType', 'Thread']
 
 _original_socket = socket.socket
@@ -138,9 +145,9 @@ class _Tls_DictStack(threading.local, _DictStack):
 
         >>> t = pwnlib.context._Tls_DictStack({})
         >>> t['key'] = 'value'
-        >>> print t
+        >>> print(t)
         {'key': 'value'}
-        >>> def p(): print t
+        >>> def p(): print(t)
         >>> thread = threading.Thread(target=p)
         >>> _ = (thread.start(), thread.join())
         {}
@@ -194,9 +201,9 @@ class Thread(threading.Thread):
         >>> context.clear()
         >>> context.update(arch='arm')
         >>> def p():
-        ...     print context.arch
+        ...     print(context.arch)
         ...     context.arch = 'mips'
-        ...     print context.arch
+        ...     print(context.arch)
         >>> # Note that a normal Thread starts with a clean context
         >>> # (i386 is the default architecture)
         >>> t = threading.Thread(target=p)
@@ -204,7 +211,7 @@ class Thread(threading.Thread):
         i386
         mips
         >>> # Note that the main Thread's context is unchanged
-        >>> print context.arch
+        >>> print(context.arch)
         arm
         >>> # Note that a context-aware Thread receives a copy of the context
         >>> t = pwnlib.context.Thread(target=p)
@@ -212,7 +219,7 @@ class Thread(threading.Thread):
         arm
         mips
         >>> # Again, the main thread is unchanged
-        >>> print context.arch
+        >>> print(context.arch)
         arm
 
     Implementation Details:
@@ -238,7 +245,13 @@ class Thread(threading.Thread):
             differently.
         """
         context.update(**self.old)
-        super(Thread, self).__bootstrap()
+        sup = super(Thread, self)
+        bootstrap = getattr(sup, '_bootstrap', None)
+        if bootstrap is None:
+            sup.__bootstrap()
+        else:
+            bootstrap()
+    _bootstrap = __bootstrap
 
 def _longest(d):
     """
@@ -252,17 +265,13 @@ def _longest(d):
     >>> pwnlib.context._longest(data) == data
     True
     >>> for i in pwnlib.context._longest(data):
-    ...     print i
+    ...     print(i)
     ccc
     bb
     a
     """
     return collections.OrderedDict((k,d[k]) for k in sorted(d, key=len, reverse=True))
 
-def TlsProperty(object):
-    def __get__(self, obj, objtype=None):
-        return obj._tls
-
 class ContextType(object):
     r"""
     Class for specifying information about the target machine.
@@ -298,7 +307,7 @@ class ContextType(object):
         >>> context.bits
         32
         >>> def nop():
-        ...   print pwnlib.asm.asm('nop').encode('hex')
+        ...   print(enhex(pwnlib.asm.asm('nop')))
         >>> nop()
         00f020e3
         >>> with context.local(arch = 'i386'):
@@ -337,10 +346,11 @@ class ContextType(object):
         'binary': None,
         'bits': 32,
         'buffer_size': 4096,
-        'cyclic_alphabet': string.ascii_lowercase,
+        'cyclic_alphabet': string.ascii_lowercase.encode(),
         'cyclic_size': 4,
         'delete_corefiles': False,
         'device': os.getenv('ANDROID_SERIAL', None) or None,
+        'encoding': 'auto',
         'endian': 'little',
         'gdbinit': "",
         'kernel': None,
@@ -392,6 +402,7 @@ class ContextType(object):
         'sparc64':   big_64,
         'thumb':     little_32,
         'vax':       little_32,
+        'none':      {},
     })
 
     #: Valid values for :attr:`endian`
@@ -420,7 +431,7 @@ class ContextType(object):
 
         All keyword arguments are passed to :func:`update`.
         """
-        self._tls = _Tls_DictStack(_defaultdict(ContextType.defaults))
+        self._tls = _Tls_DictStack(_defaultdict(self.defaults))
         self.update(**kwargs)
 
 
@@ -500,15 +511,15 @@ class ContextType(object):
             >>> context.timeout = 1
             >>> context.timeout == 1
             True
-            >>> print context.timeout
+            >>> print(context.timeout)
             1.0
             >>> with context.local(timeout = 2):
-            ...     print context.timeout
+            ...     print(context.timeout)
             ...     context.timeout = 3
-            ...     print context.timeout
+            ...     print(context.timeout)
             2.0
             3.0
-            >>> print context.timeout
+            >>> print(context.timeout)
             1.0
         """
         class LocalContext(object):
@@ -679,6 +690,7 @@ class ContextType(object):
                      ('x86_64', 'amd64'),
                      ('x86', 'i386'),
                      ('i686', 'i386'),
+                     ('armv7l', 'arm'),
                      ('armeabi', 'arm'),
                      ('arm64', 'aarch64')]
         for k, v in transform:
@@ -687,11 +699,11 @@ class ContextType(object):
                 break
 
         try:
-            defaults = ContextType.architectures[arch]
+            defaults = self.architectures[arch]
         except KeyError:
-            raise AttributeError('AttributeError: arch must be one of %r' % sorted(ContextType.architectures))
+            raise AttributeError('AttributeError: arch must be one of %r' % sorted(self.architectures))
 
-        for k,v in ContextType.architectures[arch].items():
+        for k,v in defaults.items():
             if k not in self._tls:
                 self._tls[k] = v
 
@@ -721,8 +733,8 @@ class ContextType(object):
         Even then, this doesn't matter much -- only when the the segment
         registers need to be known
         """
-        with context.local(arch=arch):
-            return context.arch
+        with self.local(arch=arch):
+            return self.arch
 
     @_validator
     def bits(self, bits):
@@ -777,6 +789,7 @@ class ContextType(object):
         self.arch   = binary.arch
         self.bits   = binary.bits
         self.endian = binary.endian
+        self.os     = binary.os
 
         return binary
 
@@ -785,7 +798,7 @@ class ContextType(object):
         """
         Target machine word size, in bytes (i.e. the size of general purpose registers).
 
-        This is a convenience wrapper around ``bits / 8``.
+        This is a convenience wrapper around ``bits // 8``.
 
         Examples:
 
@@ -798,11 +811,43 @@ class ContextType(object):
             ...
             AttributeError: bits must be > 0 (0)
         """
-        return self.bits/8
+        return self.bits // 8
     @bytes.setter
     def bytes(self, value):
         self.bits = value*8
 
+    @_validator
+    def encoding(self, charset):
+        if charset == 'auto':
+            return charset
+
+        if (  b'aA'.decode(charset) != 'aA'
+            or 'aA'.encode(charset) != b'aA'):
+            raise ValueError('Strange encoding!')
+
+        return charset
+
+    def _encode(self, s):
+        if isinstance(s, (bytes, bytearray)):
+            return s   # already bytes
+
+        if self.encoding == 'auto':
+            try:
+                return s.encode('latin1')
+            except UnicodeEncodeError:
+                return s.encode('utf-8', 'surrogateescape')
+        return s.encode(self.encoding)
+
+    def _decode(self, b):
+        if self.encoding == 'auto':
+            try:
+                return b.decode('utf-8')
+            except UnicodeDecodeError:
+                return b.decode('latin1')
+            except AttributeError:
+                return b
+        return b.decode(self.encoding)
+
     @_validator
     def endian(self, endianness):
         """
@@ -834,10 +879,10 @@ class ContextType(object):
         """
         endian = endianness.lower()
 
-        if endian not in ContextType.endiannesses:
-            raise AttributeError("endian must be one of %r" % sorted(ContextType.endiannesses))
+        if endian not in self.endiannesses:
+            raise AttributeError("endian must be one of %r" % sorted(self.endiannesses))
 
-        return ContextType.endiannesses[endian]
+        return self.endiannesses[endian]
 
 
     @_validator
@@ -894,24 +939,23 @@ class ContextType(object):
             >>> with context.local(log_file='bar.txt'):
             ...     log.debug('Hello from bar!')
             >>> log.info('Hello from foo!')
-            >>> file('foo.txt').readlines()[-3] #doctest: +ELLIPSIS
+            >>> open('foo.txt').readlines()[-3] #doctest: +ELLIPSIS
             '...:DEBUG:...:Hello!\n'
-            >>> file('foo.txt').readlines()[-2] #doctest: +ELLIPSIS
+            >>> open('foo.txt').readlines()[-2] #doctest: +ELLIPSIS
             '...:INFO:...:Hello again!\n'
-            >>> file('foo.txt').readlines()[-1] #doctest: +ELLIPSIS
+            >>> open('foo.txt').readlines()[-1] #doctest: +ELLIPSIS
             '...:INFO:...:Hello from foo!\n'
-            >>> file('bar.txt').readlines()[-1] #doctest: +ELLIPSIS
+            >>> open('bar.txt').readlines()[-1] #doctest: +ELLIPSIS
             '...:DEBUG:...:Hello from bar!\n'
         """
-        if isinstance(value, (str,unicode)):
-            modes = ('w', 'wb', 'a', 'ab')
+        if isinstance(value, (bytes, six.text_type)):
             # check if mode was specified as "[value],[mode]"
             if ',' not in value:
                 value += ',a'
             filename, mode = value.rsplit(',', 1)
             value = open(filename, mode)
 
-        elif not isinstance(value, (file)):
+        elif not hasattr(value, "fileno"):
             raise AttributeError('log_file must be a file')
 
         # Is this the same file we already have open?
@@ -979,8 +1023,8 @@ class ContextType(object):
         """
         os = os.lower()
 
-        if os not in ContextType.oses:
-            raise AttributeError("os must be one of %r" % ContextType.oses)
+        if os not in self.oses:
+            raise AttributeError("os must be one of %r" % self.oses)
 
         return os
 
@@ -1018,11 +1062,11 @@ class ContextType(object):
             ...
             AttributeError: signed must be one of ['no', 'signed', 'unsigned', 'yes'] or a non-string truthy value
         """
-        try:             signed = ContextType.signednesses[signed]
+        try:             signed = self.signednesses[signed]
         except KeyError: pass
 
         if isinstance(signed, str):
-            raise AttributeError('signed must be one of %r or a non-string truthy value' % sorted(ContextType.signednesses))
+            raise AttributeError('signed must be one of %r or a non-string truthy value' % sorted(self.signednesses))
 
         return bool(signed)
 
@@ -1046,7 +1090,7 @@ class ContextType(object):
         Can be a string or an iterable of strings.  In the latter case the first
         entry is the terminal and the rest are default arguments.
         """
-        if isinstance(value, (str, unicode)):
+        if isinstance(value, (bytes, six.text_type)):
             return [value]
         return value
 
@@ -1080,7 +1124,7 @@ class ContextType(object):
         if isinstance(proxy, str):
             proxy = (socks.SOCKS5, proxy)
 
-        if not isinstance(proxy, collections.Iterable):
+        if not isinstance(proxy, Iterable):
             raise AttributeError('proxy must be a string hostname, or tuple of arguments for socks.set_default_proxy')
 
         socks.set_default_proxy(*proxy)
@@ -1127,13 +1171,13 @@ class ContextType(object):
     def device(self, device):
         """Sets the device being operated on.
         """
+        if isinstance(device, (bytes, six.text_type)):
+            device = Device(device)
         if isinstance(device, Device):
             self.arch = device.arch or self.arch
             self.bits = device.bits or self.bits
             self.endian = device.endian or self.endian
             self.os = device.os or self.os
-        elif isinstance(device, str):
-            device = Device(device)
         elif device is not None:
             raise AttributeError("device must be either a Device object or a serial number as a string")
 
@@ -1188,12 +1232,13 @@ class ContextType(object):
             >>> cache_dir == context.cache_dir
             True
         """
-        home = os.path.expanduser('~')
+        xdg_cache_home = os.environ.get('XDG_CACHE_HOME') or \
+                         os.path.join(os.path.expanduser('~'), '.cache')
 
-        if not os.access(home, os.W_OK):
+        if not os.access(xdg_cache_home, os.W_OK):
             return None
 
-        cache = os.path.join(home, '.pwntools-cache')
+        cache = os.path.join(xdg_cache_home, '.pwntools-cache-%d.%d' % sys.version_info[:2])
 
         if not os.path.exists(cache):
             try:
@@ -1262,7 +1307,7 @@ class ContextType(object):
         if len(set(alphabet)) != len(alphabet):
             raise AttributeError("cyclic alphabet cannot contain duplicates")
 
-        return str(alphabet)
+        return alphabet.encode()
 
     @_validator
     def cyclic_size(self, size):
@@ -1387,11 +1432,39 @@ def LocalContext(function):
     """
     @functools.wraps(function)
     def setter(*a, **kw):
-        # Fast path to skip adding a Context frame
-        if not kw:
-            return function(*a)
+        with context.local(**{k:kw.pop(k) for k,v in tuple(kw.items()) if isinstance(getattr(ContextType, k, None), property)}):
+            arch = context.arch
+            bits = context.bits
+            endian = context.endian
 
-        with context.local(**{k:kw.pop(k) for k,v in kw.items() if isinstance(getattr(ContextType, k, None), property)}):
+            # Prevent the user from doing silly things with invalid
+            # architecture / bits / endianness combinations.
+            if (arch == 'i386' and bits != 32) \
+              or (arch == 'amd64' and bits != 64):
+                raise AttributeError("Invalid arch/bits combination: %s/%s" % (arch, bits))
+
+            if arch in ('i386', 'amd64') and endian == 'big':
+                raise AttributeError("Invalid arch/endianness combination: %s/%s" % (arch, endian))
+
+            return function(*a, **kw)
+    return setter
+
+def LocalNoarchContext(function):
+    """
+    Same as LocalContext, but resets arch to :const:`'none'` by default
+
+    Example:
+
+        >>> @LocalNoarchContext
+        ... def printArch():
+        ...     print(context.arch)
+        >>> printArch()
+        none
+    """
+    @functools.wraps(function)
+    def setter(*a, **kw):
+        kw.setdefault('arch', 'none')
+        with context.local(**{k:kw.pop(k) for k,v in tuple(kw.items()) if isinstance(getattr(ContextType, k, None), property)}):
             return function(*a, **kw)
     return setter
 
@@ -1408,11 +1481,19 @@ def update_context_defaults(section):
 
         default = ContextType.defaults[key]
 
-        if isinstance(default, (str, unicode, tuple, int, long, list, dict)):
+        if isinstance(default, six.string_types + six.integer_types + (tuple, list, dict)):
             value = safeeval.expr(value)
         else:
             log.warn("Unsupported configuration option %r in section %r" % (key, 'context'))
 
-        ContextType.defaults[key] = type(default)(value)
+        # Attempt to set the value, to see if it is value:
+        try:
+            with context.local(**{key: value}):
+                value = getattr(context, key)
+        except (ValueError, AttributeError) as e:
+            log.warn("Could not set context.%s=%s via pwn.conf (%s)", key, section[key], e)
+            continue
+
+        ContextType.defaults[key] = value
 
 register_config('context', update_context_defaults)
